//PIC16F628A
//Microwave timer

#define bt_up PORTA.F2
#define bt_dn PORTA.F3
#define bt_start PORTA.F4
#define bt_stop PORTA.F5
#define dr_close PORTB.F0
#define beeper PORTB.F1
#define lamp PORTA.F0
#define heat PORTA.F1

// LCD module connections
sbit LCD_RS at RB2_bit;
sbit LCD_EN at RB3_bit;
sbit LCD_D4 at RB4_bit;
sbit LCD_D5 at RB5_bit;
sbit LCD_D6 at RB6_bit;
sbit LCD_D7 at RB7_bit;

sbit LCD_RS_Direction at TRISB2_bit;
sbit LCD_EN_Direction at TRISB3_bit;
sbit LCD_D4_Direction at TRISB4_bit;
sbit LCD_D5_Direction at TRISB5_bit;
sbit LCD_D6_Direction at TRISB6_bit;
sbit LCD_D7_Direction at TRISB7_bit;
// End LCD module connections

// Software I2C connections
sbit Soft_I2C_Scl           at RA6_bit;
sbit Soft_I2C_Sda           at RA7_bit;
sbit Soft_I2C_Scl_Direction at TRISA6_bit;
sbit Soft_I2C_Sda_Direction at TRISA7_bit;
// End Software I2C connections

const char dot=':';
const sl='/';

char msg_setup[]="Set";
char msg_hour[]= "Hr:";
char msg_mins[]= "Mn:";
char msg_date[]= "Dt:";
char msg_mnt[]=  "Mo:";
char msg_year[]= "Yr:";

unsigned char *hour, *mins, *secs, *date, *mth;

signed char SetHour, SetMins, SetDate, SetMnt, SetYear, out;

char ButtTimer;

signed char TimerMin, TimerSec;

char k;

// Custom chars declare
const char msg[8][8] =
      {
          //Set 0
        {31,31,24,24,24,24,24,0},
        {31,31,3,3,3,3,3,0},
        {3,3,3,3,3,31,31,0},
        {24,24,24,24,24,31,31,0},
        {31,31,3,3,3,31,31,0},
        {31,31,24,24,24,31,31,0},
        {31,31,0,0,0,0,0,0},
        {0,0,0,0,0,31,31,0},
      };


// Write set of custom char to display memory
void CustomProg(char set) {
     char i,j;
     for (j=0; j<=7; j++) {
             Lcd_Cmd(64+(8*j));
             for (i=0; i<=7; i++) Lcd_Chr_CP(msg[j][i]);
         }
}

// Read data from DS1307
void Read_DS1307(char *hour, char *mins, char *secs, char *date, char *mth){
 char sec_t = 0;
 char mn_t = 0;
 char hr_t = 0;
 char day_t  = 0;
 char mth_t  = 0;
  Soft_I2C_Start();               // Issue start signal
  Soft_I2C_Write(0xD0);           // Address DS1307
  Soft_I2C_Write(0x00);           // Start from address 0
  Soft_I2C_Start();               // Issue repeated start signal
  Soft_I2C_Write(0xD1);           // Address DS1307 for reading R/W=1
  sec_t = Soft_I2C_Read(1); //0
  mn_t = Soft_I2C_Read(1);  //1
  hr_t = Soft_I2C_Read(1);  //2
  Soft_I2C_Read(1); //3
  day_t = Soft_I2C_Read(1); //4
  mth_t = Soft_I2C_Read(1); //5
  Soft_I2C_Read(0);//6
  Soft_I2C_Stop();                // Issue stop signal
 *secs  = Bcd2Dec(sec_t & 0x7F);
 *mins   = Bcd2Dec(mn_t  & 0x7F);
 *hour   = Bcd2Dec(hr_t  & 0x3F);
 *date    = Bcd2Dec(day_t  & 0x3F);
 *mth    = Bcd2Dec(mth_t  & 0x1F);
}

// Write data to DS1307
void Write_DS1307(char hour, char mins, char secs, char date, char mth, char year){
 char dwcnt = 1;
  Soft_I2C_Start();               // Issue start signal
  Soft_I2C_Write(0xD0);           // Address DS1307
  Soft_I2C_Write(0x00);           // Start from address 0
  Soft_I2C_Write(Dec2Bcd(secs & 0x7F));
  Soft_I2C_Write(Dec2Bcd(mins & 0x7F));
  Soft_I2C_Write(Dec2Bcd(hour & 0x3F));
  Soft_I2C_Write(Dec2Bcd(dwcnt));
  Soft_I2C_Write(Dec2Bcd(date & 0x3F));
  Soft_I2C_Write(Dec2Bcd(mth & 0x1F));
  Soft_I2C_Write(Dec2Bcd(year));
  Soft_I2C_Write(0x10);             // 1Hz squarewave output pin
  Soft_I2C_Stop();
}

// WriteByte
void WriteByte(char row, char col, char number){
   char *bytetext = "00";
   bytetext[0] = (number/10)%10 + 48;             // Extract tens digit
   bytetext[1] =  number%10     + 48;             // Extract ones digit
   Lcd_Out(row, col, bytetext);
}

// WriteBig
void WriteBig(char row, char col, char number){
   char ob[8];
   char dig1, dig2;
   dig1 = (number/10)%10;             // Extract tens digit
   dig2 =  number%10;             // Extract ones digit
   switch(dig1){
     case 0: {ob[0]=0; ob[1]=1; ob[4]=3; ob[5]=2;} break;
     case 1: {ob[0]=1; ob[1]=' '; ob[4]=2; ob[5]=7;} break;
     case 2: {ob[0]=6; ob[1]=4; ob[4]=5; ob[5]=7;} break;
     case 3: {ob[0]=6; ob[1]=4; ob[4]=7; ob[5]=2;} break;
     case 4: {ob[0]=3; ob[1]=2; ob[4]=' '; ob[5]=1;} break;
     case 5: {ob[0]=5; ob[1]=6; ob[4]=7; ob[5]=4;} break;
     case 6: {ob[0]=0; ob[1]=6; ob[4]=5; ob[5]=4;} break;
     case 7: {ob[0]=6; ob[1]=1; ob[4]=' '; ob[5]=1;} break;
     case 8: {ob[0]=5; ob[1]=4; ob[4]=3; ob[5]=2;} break;
     case 9: {ob[0]=5; ob[1]=4; ob[4]=7; ob[5]=2;} break;
   }
   switch(dig2){
     case 0: {ob[2]=0; ob[3]=1; ob[6]=3; ob[7]=2;} break;
     case 1: {ob[2]=1; ob[3]=' '; ob[6]=2; ob[7]=7;} break;
     case 2: {ob[2]=6; ob[3]=4; ob[6]=5; ob[7]=7;} break;
     case 3: {ob[2]=6; ob[3]=4; ob[6]=7; ob[7]=2;} break;
     case 4: {ob[2]=3; ob[3]=2; ob[6]=' '; ob[7]=1;} break;
     case 5: {ob[2]=5; ob[3]=6; ob[6]=7; ob[7]=4;} break;
     case 6: {ob[2]=0; ob[3]=6; ob[6]=5; ob[7]=4;} break;
     case 7: {ob[2]=6; ob[3]=1; ob[6]=' '; ob[7]=1;} break;
     case 8: {ob[2]=5; ob[3]=4; ob[6]=3; ob[7]=2;} break;
     case 9: {ob[2]=5; ob[3]=4; ob[6]=7; ob[7]=2;} break;
   }
   Lcd_Chr(row,col,ob[0]);
   Lcd_Chr(row,col+1,ob[1]);
   Lcd_Chr(row+1,col,ob[4]);
   Lcd_Chr(row+1,col+1,ob[5]);
 
   Lcd_Chr(row,col+2,ob[2]);
   Lcd_Chr(row,col+3,ob[3]);
   Lcd_Chr(row+1,col+2,ob[6]);
   Lcd_Chr(row+1,col+3,ob[7]);
}

// changing value "out"
void adjust(char min, char max){
    do{
       if (bt_up == 1) out++;
          if (out>max) out=min;
       if (bt_dn == 1) out--;
          if (out<min) out=max;
       WriteByte(2,8,out);
       delay_ms(500);
    }
    while(bt_start == 0);
}

// setup clock
void setuptime(){
  char txt[3];
  Lcd_Cmd(_LCD_CLEAR);
  Lcd_Out(1,1,msg_setup);
  while (bt_stop == 1){};

     Lcd_Out(2,2,msg_hour);
     out=Hour;
     adjust(0,23);
     SetHour=out;
  delay_ms(10);
  while (bt_start == 1){};

     Lcd_Out(2,2,msg_mins);
     out=Mins;
     adjust(0,59);
     SetMins=out;
  delay_ms(10);
  while (bt_start == 1){};

     Lcd_Out(2,2,msg_date);
     out=Date;
     adjust(1,31);
     SetDate=out;
  delay_ms(10);
  while (bt_start == 1){};

     Lcd_Out(2,2,msg_mnt);
     out=mth;
     adjust(1,12);
     SetMnt=out;
  delay_ms(10);
  while (bt_start == 1){};

     Lcd_Out(2,2,msg_year);
     out=13;
     adjust(1,99);
     SetYear=out;

  Write_DS1307(SetHour,SetMins,0,SetDate,SetMnt,SetYear);
  Lcd_Cmd(_LCD_CLEAR);
}

// running heat
void Run(){
  char k, newsec;
  char oldsec=99;
  lamp=1;
  delay_ms(1000);
  heat=1;
  while ((TimerSec>0)||(TimerMin>0)) 
  {
   Read_DS1307(&hour, &mins, &secs, &date, &mth);
   newsec = secs;
   if (oldsec != newsec)
     {
      TimerSec--;
      if (TimerSec<0) {TimerSec=59; TimerMin--;}
      oldsec=newsec;
     }
   WriteBig(1,2,TimerMin);
   WriteBig(1,7,TimerSec);
   if ((dr_close == 0)||(bt_stop)) break;
  }
  heat=0;
  if ((dr_close)&&(bt_stop == 0)) 
      {
        for(k=0;k<3;k++)
         {
           beeper=1;
           delay_ms(500);
           beeper=0;
           delay_ms(500);
         }
      }
  lamp=0;
}

// setup heat timer
void TimerSet(){
  bit OldUp, OldDn;
  Lcd_Cmd(_LCD_CLEAR);
  Lcd_Chr(1,6,0x5F);
  TimerMin=0;
  TimerSec=0;
  do {
    if (bt_up==0) OldUp=0;
    if ((bt_up==1)&&(OldUp==0))
      { 
      TimerSec+=30;
      if (TimerSec>30) {TimerSec=0; TimerMin++;}
      OldUp=1;
      delay_ms(10);
      }
    if (bt_dn==0) OldDn=0;
    if ((bt_dn==1)&&(OldDn==0))
      {
      TimerSec-=30;
      if (TimerSec<0) {TimerSec=30; TimerMin--;}
      if (TimerMin<0) TimerMin=0;
      OldDn=1;
      delay_ms(10);
      }
    WriteBig(1,2,TimerMin);
    WriteBig(1,7,TimerSec);
    if ((bt_start==1)&&((TimerSec>0)||(TimerMin>0))) Run();
  } while ((bt_stop == 0)&&(dr_close));
  
  Lcd_Cmd(_LCD_CLEAR);
}

// MAIN
void main() {
     INTCON=0;
     CMCON=7;
     TRISA=0b11111100;
     TRISB=0b11111101;
     beeper=0;
     Lcd_Init();                        // Initialize LCD
     Lcd_Cmd(_LCD_CLEAR);               // Clear display
     Lcd_Cmd(_LCD_CURSOR_OFF);          // Cursor off
     CustomProg(0);
     Butttimer=0;
     while(1)
        {
           Read_DS1307(&hour, &mins, &secs, &date, &mth);
           WriteByte(1,1,hour);
           Lcd_Chr(1,3,dot);
           WriteByte(1,4,mins);
           Lcd_Chr(1,6,dot);
           WriteByte(1,7,secs);
           WriteByte(2,1,date);
           Lcd_Chr(2,3,sl);
           WriteByte(2,4,mth);

           if (bt_stop == 0) ButtTimer = 0; else Butttimer++;
           if (ButtTimer == 0x7F) SetupTime();
           if (((bt_up == 1)||(bt_dn == 1))&&(dr_close)) TimerSet();
        }
     }